import logging
import pprint

from django.core.exceptions import ValidationError
from django.core.validators import EmailValidator

logger = logging.getLogger(__name__)


class RecipientFilter:
    """Фильтр email на основании их валидности"""

    def __init__(self):
        self.validator = EmailValidator()

    def filter_emails(self, recipients: list) -> list:
        """Возвращает список валидных email, невалидные исключаются"""
        valid_list = []
        for recipient in recipients:
            try:
                self.validator(recipient.strip())
            except (ValidationError, UnicodeError) as ex:
                logger.exception(
                    f'Can`t send email message to {recipient}.'
                    f' {pprint.saferepr(ex)}')
                continue
            valid_list.append(recipient)
        return valid_list



import abc

from django.template import Context, Template


class BaseMail(abc.ABC):
    """Базовый класс отправления email"""

    def __init__(self, **kwargs):
        context = self._prepare_context(**kwargs)
        self._text = self._generate_text(context)
        self._text += self.footer
        self._subject = self._generate_subject(context)

    @property
    def text(self) -> str:
        """Текстовое содержимое письма"""
        return self._text

    @property
    def subject(self) -> str:
        """Тема письма"""
        return self._subject

    @property
    @abc.abstractmethod
    def template(self) -> Template:
        """Шаблон письма"""
        pass

    @property
    @abc.abstractmethod
    def subject_template(self) -> Template:
        """Шаблон заголовка письма"""
        pass

    @property
    def footer(self) -> str:
        """Стандартное окончание письма"""
        return ''

    def _prepare_context(self, **kwargs) -> Context:
        """Подготовка контекста для шаблона

        По-умолчанию - передача kwargs из конструктора.
        Переопределяется в зависимости от требуемого функционала
        """
        return Context(kwargs)

    def _generate_text(self, context):
        """Подстановка данных в шаблон тела письма"""
        if not isinstance(self.template, Template):
            raise ValueError('template must be a Template')

        return self.template.render(context)

    def _generate_subject(self, context):
        """Подстановка данных в шаблон заголовка письма"""
        if not isinstance(self.template, Template):
            raise ValueError('subject template must be a Template')

        return self.subject_template.render(context)



from smtplib import SMTPException
from typing import Optional

from django.conf import settings
from django.core.mail import EmailMultiAlternatives

from project.celery import app


@app.task(bind=True, max_retries=settings.CELERY_MAX_RETRIES_COUNT)
def send_email(self, subject: str, recipients: list[str], body: str,
               backoff: Optional[int] = None) -> None:
    """Celery task отправления email"""
    try:
        msg = EmailMultiAlternatives(
            subject=subject, body=body,
            to=recipients, from_email=settings.EMAIL_HOST_USER,
        )
        msg.mixed_subtype = 'related'
        msg.attach_alternative(body, 'text/html')
        msg.send()
    except SMTPException as ex:
        countdown = backoff * pow(2, self.request.retries) if backoff else None
        raise self.retry(exc=ex, countdown=countdown)
